@@ -229,7 +229,7 @@ void QXcbBackingStoreImage::resize(const QSize &size)
                                    m_xcb_format->bits_per_pixel,
                                    0, byteOrder,
                                    XCB_IMAGE_ORDER_MSB_FIRST,
-                                   0, ~0, 0);
+                                   nullptr, ~0, nullptr);
 
     const size_t segmentSize = imageDataSize(m_xcb_image);
 
@@ -412,13 +412,13 @@ bool QXcbBackingStoreImage::createSystemVShmSegment(xcb_connection_t *c, size_t
         return false;
     }
 
-    void *addr = shmat(id, 0, 0);
+    void *addr = shmat(id, nullptr, 0);
     if (addr == (void *)-1) {
         qCWarning(lcQpaXcb, "shmat() failed (%d: %s) for id %d", errno, strerror(errno), id);
         return false;
     }
 
-    if (shmctl(id, IPC_RMID, 0) == -1)
+    if (shmctl(id, IPC_RMID, nullptr) == -1)
         qCWarning(lcQpaXcb, "Error while marking the shared memory segment to be destroyed");
 
     const auto seg = xcb_generate_id(c);
@@ -641,17 +641,17 @@ void QXcbBackingStoreImage::flushPixmap(const QRegion &region, bool fullRegion)
     xcb_subimage.bit_order = m_xcb_image->bit_order;
 
     const bool needsByteSwap = xcb_subimage.byte_order != m_xcb_image->byte_order;
+    // Ensure that we don't send more than maxPutImageRequestDataBytes per request.
+    const auto maxPutImageRequestDataBytes = connection()->maxRequestDataBytes(sizeof(xcb_put_image_request_t));
 
     for (const QRect &rect : region) {
-        // We must make sure that each request is not larger than max_req_size.
-        // Each request takes req_size + m_xcb_image->stride * height bytes.
-        static const uint32_t req_size = sizeof(xcb_put_image_request_t);
-        const uint32_t max_req_size = xcb_get_maximum_request_length(xcb_connection());
-        const int rows_per_put = (max_req_size - req_size) / m_xcb_image->stride;
+        const quint32 stride = round_up_scanline(rect.width() * m_qimage.depth(), xcb_subimage.scanline_pad) >> 3;
+        const int rows_per_put = maxPutImageRequestDataBytes / stride;
 
         // This assert could trigger if a single row has more pixels than fit in
-        // a single PutImage request. However, max_req_size is guaranteed to be
-        // at least 16384 bytes. That should be enough for quite large images.
+        // a single PutImage request. In the absence of the BIG-REQUESTS extension
+        // the theoretical maximum lengths of maxPutImageRequestDataBytes can be
+        // roughly 256kB.
         Q_ASSERT(rows_per_put > 0);
 
         // If we upload the whole image in a single chunk, the result might be
@@ -666,9 +666,10 @@ void QXcbBackingStoreImage::flushPixmap(const QRegion &region, bool fullRegion)
         while (height > 0) {
             const int rows = std::min(height, rows_per_put);
             const QRect subRect(x, y, width, rows);
-            const quint32 stride = round_up_scanline(width * m_qimage.depth(), xcb_subimage.scanline_pad) >> 3;
             const QImage subImage = native_sub_image(&m_flushBuffer, stride, m_qimage, subRect, needsByteSwap);
 
+            Q_ASSERT(static_cast<size_t>(subImage.sizeInBytes()) <= maxPutImageRequestDataBytes);
+
             xcb_subimage.width = width;
             xcb_subimage.height = rows;
             xcb_subimage.data = const_cast<uint8_t *>(subImage.constBits());
@@ -709,9 +710,10 @@ void QXcbBackingStoreImage::put(xcb_drawable_t dst, const QRegion &region, const
     Q_ASSERT(!m_clientSideScroll);
 
     ensureGC(dst);
-    setClip(region);
 
     if (hasShm()) {
+        setClip(region); // Clip in window local coordinates
+
         // Copy scrolled area on server-side from pixmap to window
         const QRegion scrolledRegion = m_scrolledRegion.translated(-offset);
         for (const QRect &rect : scrolledRegion) {
@@ -732,7 +734,15 @@ void QXcbBackingStoreImage::put(xcb_drawable_t dst, const QRegion &region, const
         const QRect bounds = region.boundingRect();
         const QPoint target = bounds.topLeft();
         const QRect source = bounds.translated(offset);
-        flushPixmap(region);
+
+        // First clip in backingstore-local coordinates, and upload
+        // the changed parts of the backingstore to the server.
+        setClip(source);
+        flushPixmap(source);
+
+        // Then clip in window local coordinates, and copy the updated
+        // parts of the backingstore image server-side to the window.
+        setClip(region);
         xcb_copy_area(xcb_connection(),
                       m_xcb_pixmap,
                       dst,
@@ -779,7 +789,7 @@ QXcbBackingStore::~QXcbBackingStore()
 QPaintDevice *QXcbBackingStore::paintDevice()
 {
     if (!m_image)
-        return 0;
+        return nullptr;
     return m_rgbImage.isNull() ? m_image->image() : &m_rgbImage;
 }
 
@@ -1035,7 +1045,7 @@ void QXcbSystemTrayBackingStore::recreateImage(QXcbWindow *win, const QSize &siz
     xcb_create_pixmap(xcb_connection(), 32, m_xrenderPixmap, screen->root(), size.width(), size.height());
 
     m_xrenderPicture = xcb_generate_id(xcb_connection());
-    xcb_render_create_picture(xcb_connection(), m_xrenderPicture, m_xrenderPixmap, m_xrenderPictFormat, 0, 0);
+    xcb_render_create_picture(xcb_connection(), m_xrenderPicture, m_xrenderPixmap, m_xrenderPictFormat, 0, nullptr);
 
     // XRender expects premultiplied alpha
     if (m_image)
@@ -1076,7 +1086,7 @@ void QXcbSystemTrayBackingStore::initXRenderMode()
 
     m_windowPicture = xcb_generate_id(conn);
     xcb_void_cookie_t cookie =
-            xcb_render_create_picture_checked(conn, m_windowPicture, platformWindow->xcb_window(), vfmt->format, 0, 0);
+            xcb_render_create_picture_checked(conn, m_windowPicture, platformWindow->xcb_window(), vfmt->format, 0, nullptr);
     xcb_generic_error_t *error = xcb_request_check(conn, cookie);
     if (error) {
         qWarning("QXcbSystemTrayBackingStore: Failed to create Picture with format %x for window %x, error code %d",
